//
//  ENCustomPopoverBackgroundView.h
//  ENPopover
//
//  Created by Matt Sarnoff on 4/13/12.
//  Copyright (c) 2012 Evernote Corp. All rights reserved.
//
//  A popover root view that lets you draw a custom popover background.
//  (Drawing is currently limited to tinting the background.)
//  Works with NSPopover and MAAttachedWindow.
//
//  This is an abstract class. To use it, subclass it and implement drawRect:.
//  Any drawing should use NSCompositeSourceAtop to preserve the shape of the popover.
//
//  Setup:
//  - Make this view the root view of the popover.
//  - Create another NSView inside of it, with identical bounds, and connect it
//    to mContentView.
//
//  Notes:
//  - Does not work with resizable popovers (yet)

#import <Cocoa/Cocoa.h>

@interface ENCustomPopoverBackgroundView : NSView
{
  IBOutlet NSView *mContentView;
  NSSize mInnerMargins;
  NSColor *mBackgroundColor;
}

/**
 * The recommended way to create an ENCustomPopoverBackgroundView programmatically,
 * Creates mContentView and installs it as a subview.
 */
- (id)initWithContentViewAndFrame:(NSRect)frame;


/**
 * Inner margins
 */
@property (assign) NSSize innerMargins;


/**
 * Uses a hack to override the background color.
 * Set to nil to use the default background behavior.
 */
@property (retain) NSColor *backgroundColor;


/**
 * Returns the inner content view.
 * When creating an ENCustomPopoverBackgroundView programmatically,
 * any controls/subviews should be added to this view.
 */
@property (readonly) NSView *contentView;

@end
