//
//  ENLocationManager.h
//  EverNote
//
//  Created by Zdzislaw Losvik on 30/08/2012.
//  Copyright (c) 2012 Evernote. All rights reserved.
//

#import <Foundation/Foundation.h>

#import <CoreLocation/CoreLocation.h>

typedef enum
{
  ENLocationManagerStatusOk             = 0,
  ENLocationManagerStatusDenied         = 1,
  ENLocationManagerStatusUnavailable    = 2,
  ENLocationManagerStatusTimeout        = 3,
  ENLocationManagerStatusUnknown        = 4,
} ENLocationManagerStatus;

typedef void (^ENLocationResponse)(CLLocation *location, ENLocationManagerStatus status, NSError *error);

/**
 * ENLocationManager provides access to the current user location.
 */
@interface ENLocationManager : NSObject <CLLocationManagerDelegate>
{
  NSMutableArray          *mPendingRequests;
  CLLocationManager       *mLocationManager;
}

/**
 * Get shared location manager
 */
+ (ENLocationManager *)sharedLocationManager;

/**
 * Can user location be obtained?
 */
- (BOOL)canObtainLocations;

/**
 * Obtain an error describing the status code.
 */
+ (NSError *)errorForStatus:(ENLocationManagerStatus)statusCode;

/**
 * Return the last known location. Returns nil if location hasn't been requested
 * so far or it is not possible to obtain it.
 */
- (CLLocation *)lastKnownLocation;

/**
 * Request user location from system.
 * @param locationResponse  Block to be executed once the location is known. May be called immediatelly.
 * @param context           Context is used for batch cancellation purpose. It is not retained.
 */
- (void)locationWithResponse:(ENLocationResponse)locationResponse context:(id)context;

/**
 * Cancel all requests associated with given context
 * @param context           Context is used when requests where made.
 */
- (void)cancelRequestsForContext:(id)context;

/**
 * Check if there are any pending requests for the context
 * @param context           Context is used when requests where made.
 */
- (BOOL)hasRequestsForContext:(id)context;

@end
