//
//  ENCPClean.js
//  CoverPage
//
//  Created by Zdzislaw Losvik on 09/04/2013.
//  Copyright (c) 2013 Evernote. All rights reserved.
//

//"use strict";

function ENCPClean() {
}

ENCPClean.prototype = {

  COLOR_DESATURATION_FACTOR: 0.6,

  desaturateColor : function (color, amount) {
    var oldColor = this.convertColorToRGB(color);
    var intensity = 0.3 * oldColor.R + 0.59 * oldColor.G + 0.11 * oldColor.B;
    var newColor = {
      R : intensity * amount + oldColor.R * (1 - amount),
      G : intensity * amount + oldColor.G * (1 - amount),
      B : intensity * amount + oldColor.B * (1 - amount)
    };
    var colorValue = ((newColor.R << 16) | (newColor.G << 8) | newColor.B).toString(16);
    return "#" + ('000000' + colorValue).slice(-6);
  },

  convertColorToRGB: function(color) {
    if (color == 'black') {
      color = "000000";
    }
    if (color[0] == "#") {
      color = color.slice(1);
    }
    var rgbRegex = /\s*rgb\s*\(\s*(\d+)\s*,\s*(\d+)\s*,\s*(\d+)\s*\)/g;
    var rgbMatch = rgbRegex.exec(color);
    if (rgbMatch) {
      var color = {
        R : parseInt(rgbMatch[1]),
        G : parseInt(rgbMatch[2]),
        B : parseInt(rgbMatch[3])
      };
    } else {
      var colorInt = parseInt(color, 16);
      var color = {
        R : (colorInt >> 16) & 0xFF,
        G : (colorInt >> 8) & 0xFF,
        B : colorInt & 0xFF
      };
    }
    return color;
  },

  isBlackColor: function(colorString) {
    var color = this.convertColorToRGB(colorString);
    return color.R == 0 && color.G == 0 && color.B == 0;
  },

  computeMediaLayout : function (numberOfMedia) {
    if (!numberOfMedia) {
      return null;
    }
    if (numberOfMedia == 1) {
      return [1];
    }

    var blockCount = Math.floor((numberOfMedia + 3) / 5);
    var lastBlockMediaCount = numberOfMedia - 5 * (blockCount - 1);
    var layoutArray = [];
    var currentMediaCount = !(lastBlockMediaCount % 3) ? 3 : 2;
    while (blockCount > 1) {
      layoutArray.push(currentMediaCount);
      layoutArray.push((currentMediaCount == 3) ? 2 : 3);
      blockCount--;
    }

    currentMediaCount = Math.min(lastBlockMediaCount, currentMediaCount);
    layoutArray.push(currentMediaCount);
    lastBlockMediaCount -= currentMediaCount;
    if (lastBlockMediaCount) {
      layoutArray.push(lastBlockMediaCount);
    }

    return layoutArray;
  },

  isEmptyTextNode : function (node) {
    if (node.nodeType != Node.TEXT_NODE) {
      return false;
    }

    var nodeText = node.nodeValue;
    nodeText = (nodeText !== null) ? nodeText.replace(/^\s*/, '').replace(/\s*$/, '') : '';
    return nodeText.length === 0;
  },

  findMediaNodes : function (rootElement) {
    return $(rootElement).find('img, embed').filter(function () {
      var parentChain = $(this).parentsUntil(rootElement);
      return !parentChain.filter("table, td, tr").length
    }).toArray();
  },

  canSkipElementsBetweenMediaNodes : function (rootElement, firstElement, lastElement) {
    var self = this;

    function canSkipElement(element) {
      if (self.isEmptyTextNode(element)) {
        return true;
      }
      return ['div', 'p', 'br', 'span'].indexOf(element.nodeName.toLowerCase()) != -1;
    }

    function nextElement(element) {
      if (element.hasChildNodes()) {
        return element.childNodes[0];
      }
      if (element.nextSibling) {
        return element.nextSibling;
      }
      while (element.parentElement && element.parentElement != rootElement) {
        if (element.parentElement.nextSibling) {
          return element.parentElement.nextSibling;
        }
        element = element.parentElement;
      }

      throw new Error("There are no more elements to search");
    }

    if (firstElement == lastElement) {
      return false;
    }

    for (var currentElement = nextElement(firstElement);
         currentElement != lastElement;
         currentElement = nextElement(currentElement)) {
      if (!canSkipElement(currentElement)) {
        return false;
      }
    }
    return true;
  },

  fixMediaLayout : function (rootElement) {
    var mediaNodes = this.findMediaNodes(rootElement);
    $(mediaNodes).each(function() {
      $(this).wrap("<div class=\"coverPageMedia\"></div>").wrap("<div class=\"coverPageMediaInner\"/>");
      $(this).attr("onclick", "window.notePresenter.onImageClick(this)");
    });
  },

  fixElementsStyle : function (rootElement) {
    var self = this;
    $(rootElement).find("[style]").each(function() {
      var element = $(this);
      var fontFamily = element.css("font-family");
      if (fontFamily) {
        element.css("font-family", "");
      }
      // CP-227 - Use style attribute instead of Jquery css function, which converts the size to
      // pixels resulting in different results on different hardware (retina/non retina)
      var fontSize = this.style.fontSize;
      if (fontSize) {
        element.css("font-size", "");
        element.addClass(self.computeFontClassForSize(fontSize));
      }
      element.css("font-size", "");
      var color = element.css("color");
      if (color) {
        if (self.isBlackColor(color)) {
          element.css("color", "");
        } else {
          element.css("color", self.desaturateColor(color, self.COLOR_DESATURATION_FACTOR));
        }
      }
    });
  },

  computeFontClassForSize: function(sizeString) {
    if (sizeString.search(/px|pt/gi) != -1) {
      var size = parseInt(sizeString.replace(/px/gi));
    } else if (sizeString.search(/em/gi) != -1) {
      var size = Math.ceil(parseFloat(sizeString.replace(/em/gi)) * 12);
    } else if (sizeString.search(/xx-small/gi) != -1) {
      var size = 8;
    } else if (sizeString.search(/x-small/gi) != -1) {
      var size = 10;
    } else if (sizeString.search(/small/gi) != -1) {
      var size = 12;
    } else if (sizeString.search(/medium/gi) != -1) {
      var size = 14;
    } else if (sizeString.search(/large/gi) != -1) {
      var size = 18;
    } else if (sizeString.search(/x-large/gi) != -1) {
      var size = 24;
    } else if (sizeString.search(/xx-large/gi) != -1) {
      var size = 36;
    } else {
      var size = parseInt(sizeString);
      switch (size) {
        case 1: size = 10; break;
        case 2: size = 13; break;
        case 3: size = 16; break;
        case 4: size = 18; break;
        case 5: size = 24; break;
        case 6: size = 32; break;
        case 7: size = 48; break;
        default: size = 13; break;
      }
    }
    if (size <= 9) {
      return "small-font";
    } else if (size <= 14) {
      return "medium-font";
    } else {
      return "large-font";
    }
  },

  fixFontElements: function(rootElement) {
    var self = this;
    $(rootElement).find("font").each(function() {
      var element = $(this);
      var size = element.attr("size");
      if (size) {
        element.removeAttr("size");
        element.addClass(self.computeFontClassForSize(size));
      }
      if (element.attr("face")) {
        element.removeAttr("face");
      }
    });
  },

  fixElementsColor : function (rootElement) {
    var self = this;
    $(rootElement).find("[color]").each(function() {
      var element = $(this);
      var color = element.attr("color");
      if (self.isBlackColor(color)) {
        element.removeAttr("color");
      } else {
        element.attr("color", self.desaturateColor(color, self.COLOR_DESATURATION_FACTOR));
      }
    });
  },

  fixEmbeddedMedia : function (rootElement) {
    $(rootElement).find("embed[id^='en-media']").replaceWith( function () {
      var embed = $(this);
      var img = $("<img class='en-media'/>");
      img.attr("src", "http://encoverpage.localhost/" + embed.attr("src"));
      img.addClass(embed.attr("class"));
      img.attr("onclick", embed.attr("onclick"));
      return img;
    });
  },

  appendDocumentShadows: function(rootElement) {
    $(rootElement).find("img.en-media[src$='pdf']").each(function() {
      var docImage = $(this);
      docImage.addClass("en-document");
      docImage.wrap($("<div class='en-media-document'></div>"));
      var theContainer = docImage.parent();
      theContainer.append($("<div class='en-shadow en-first-shadow'></div>"));
      theContainer.append($("<div class='en-shadow en-second-shadow'></div>"));
    });
  },

  postprocessWebContent: function (rootElement) {
    this.convertEmbeddedCheckboxesToHTML($(rootElement));
    this.fixMediaLayout(rootElement);
    this.fixEmbeddedMedia(rootElement);
  },

  postprocessNoteContent: function (rootElement, noteTitle, updated) {
    this.convertEmbeddedCheckboxesToHTML(rootElement);
    this.fixFontElements(rootElement);
    this.fixElementsStyle(rootElement);
    this.fixElementsColor(rootElement);
    this.fixMediaLayout(rootElement);
    this.fixEmbeddedMedia(rootElement);
    this.styleOrderedLists(rootElement);
    this.insertNoteTitle(rootElement, noteTitle, updated)
    this.appendDocumentShadows(rootElement);
  },

  /**
   * @param rootElement: the element and its children to be changed
   * @param noteTitle: the note title
   * @param updated: date the note was last updated
   * @param source: the source of the note (mail etc)
   * @param contentClass: special note like hello, food
   */
  postprocess : function (rootElement, noteTitle, updated, source, contentClass) {
    $("html").attr("en-source", source);
    $("html").attr("en-content-class", contentClass);
    if (contentClass || source == 'mail.smtp' || source == 'web.clip') {
      this.postprocessWebContent(rootElement);
      $("html").addClass("websource");
    } else {
      this.postprocessNoteContent(rootElement, noteTitle, updated);
      $("html").addClass("notesource");
    }
  },

  protectCheckboxesFromClearly: function(rootElement) {
    rootElement.find("embed[type='evernote/x-todo']").addClass("protectedFromClearly");
  },

  convertEmbeddedCheckboxesToHTML : function(rootElement) {
    $(rootElement).find("embed[type='evernote/x-todo']").replaceWith(function() {
      var checkbox = $("<div class='checkbox'></div>");
      if ($(this).attr("todo-checked") == 'true') {
        checkbox.addClass("checked");
      }
      return checkbox;
    });
  },

  styleOrderedLists: function(rootElement) {
    $(rootElement).find("ol").each(function() {
      var ol = $(this);
      var start = ol.attr("start");
      start = start ? parseInt(start) : 1;
      var realIndex = 0;
      ol.children("li").each(function(index) {
        var liElement = $(this);
        liElement.wrapInner("<span class='content'></span>");
        var indexAttr = liElement.attr("value");
        realIndex = (indexAttr && indexAttr != 0) ? indexAttr : (index + start);
        var counter = $("<span class=\"counter\">" + realIndex + "</span>");
        liElement.prepend(counter);
      });
      if (realIndex < 10) {
        ol.addClass("onedigit");
      } else if (realIndex < 100) {
        ol.addClass("twodigits");
      } else if (realIndex < 1000) {
        ol.addClass("threedigits");
      } else if (realIndex < 10000) {
        ol.addClass("fourdigits");
      } else {
        ol.addClass("fivedigitsormore");
      }
    });
  },

  insertNoteTitle: function(rootElement, title, date) {
    if (!title || !title.length) {
      return;
    }
    var titleDiv = $(
        "<div class='noteTitle'>" +
          "<div class='titleContainer'>" +
          "<h1 class='title'>" + title + "</h1>" +
          "<hr/>" +
          "<h2 class='date'>" + date + "</h2>" +
        "</div>" +
    "</div>");
    $(rootElement).prepend(titleDiv);
  },

  clearlyfyElement: function(rootElement, completion) {
    window.jQueryForClearlyComponent = window.jQuery;
    window.injectClearly();
    window.ClearlyComponent.getContentElementAndHTML(rootElement.get(), completion);
  },

  clearlyfy: function(rootElement, completion) {
    if (!rootElement) {
      rootElement = $(document.body);
    }
    this.protectCheckboxesFromClearly(rootElement);
    this.clearlyfyElement(rootElement, function(result) {
      if (completion) {
        completion(result);
      }
    });
  },

  simplify: function(rootElement, completion) {
    if (!rootElement) {
      rootElement = $(document.body);
    }
    var result = EN.createNewSimpleTextElement(rootElement.get(0));
    completion({_html: $(result).html()});
  },

  original: function(rootElement, completion) {
    completion({_html: $(rootElement).html()});
  }

};

//window.addEventListener('load', function () {
//  console.log("Cleaning up note");
//  var cp = new ENCPClean();
//  cp.postprocess(document.body, null, null);
//}, false);

