//
//  ENCPPresenter.js
//  CoverPage
//
//  Created by Zdzislaw Losvik on 17/04/2013.
//  Copyright (c) 2013 Evernote. All rights reserved.
//

function ENCPPresenter() {
}

ENCPPresenter.prototype = {

  REFERENCE_SCREEN_RESOLUTION: 1440,
  REFERENCE_FONT_SIZE_IN_PX:30,
  STANDARD_EM_SIZE_IN_PX:16,
  ADDITIONAL_ZOOM_PERCENT: 10,

  onMediaClick : function (src, mediaFrame) {
    //noinspection JSUnresolvedVariable
    if (ENCPPresentation == null) {
      console.log("Cannot access host");
      return;
    }

    var parameters = {'src': src, 'mediaFrame': mediaFrame};
    //noinspection JSUnresolvedVariable,JSUnresolvedFunction
    ENCPPresentation.didClickMediaWithParameters(JSON.stringify(parameters));
  },

  elementFrame : function (element) {
    var $element = $(element);
    var offset = $element.offset();
    return {X: offset.left, Y: offset.top, Width: $element.width(), Height: $element.height()};
  },

  onImageClick : function (imageNode) {
    var src = imageNode.getAttribute('src');
    src = this.stripQuickLoopPathPrefix(src);
    var mediaFrame = this.elementFrame(imageNode);
    this.onMediaClick(src, mediaFrame);
  },

  prepareExitGalleryModeForMediaAtPath : function(mediaPath, rootElement) {
    if (!rootElement) {
      rootElement = document.body;
    }
    var $imageNode = $(rootElement).find("[src$='" + mediaPath + "']");
    if ($imageNode.length == 0) {
      return null;
    }
    if (!this.isElementOnScreen($imageNode.get())) {
      this.scrollIntoView($imageNode.get(0));
    }
    var mediaFrame = this.elementFrame($imageNode.get())
    return JSON.stringify({'mediaFrame': mediaFrame});
  },

  stripQuickLoopPathPrefix : function(path) {
    var quickLookPrefix = "http://encoverpage.localhost/";
    if (path.indexOf(quickLookPrefix) === 0) {
      return path.slice(quickLookPrefix.length);
    }
    return path;
  },

  updateLayoutForMediaItem : function (mediaItem) {
    var $mediaItem = $(mediaItem);
    var $row = $mediaItem.parents(".coverPageMediaRow");
    var heightDiff = $row.height() - $mediaItem.height();
    $mediaItem.css("top", Math.min(heightDiff / 2, 0));
  },

  disableAnchorTags: function () {
    $("a").on("click", function(event){
      event.preventDefault();
      // We may want to show something later to the user here
    });
  },

  registerResizeListener: function() {
    var self = this;
    $(window).on("resize", function(event) {
      self.scalePresentation();
      self.updateImageSizes();
    });
  },

  scalePresentation: function() {
    if ($("html").hasClass("websource")) {
      var percent = window.innerWidth * 100 / this.REFERENCE_SCREEN_RESOLUTION + this.ADDITIONAL_ZOOM_PERCENT;
      $("html").css("zoom", "" + percent + "%" );
    } else {
      var percent = (this.REFERENCE_FONT_SIZE_IN_PX * 100 / this.STANDARD_EM_SIZE_IN_PX) * window.innerWidth / this.REFERENCE_SCREEN_RESOLUTION;
      $("html").css("font-size", "" + percent + "%" );
    }
  },

  updateImageSizes: function() {
    var self = this;
    $(".coverPageMedia img").each(function() {
      self.updateImageSize(this);
    });
  },

  updateImageSize: function(element) {
    var media = $(element).parents(".coverPageMedia");
    if ($("html").width() < element.naturalWidth) {
      media.addClass("fullWidth");
    } else {
      media.removeClass("fullWidth");
    }
    media.addClass("loaded");
  },

  registerImageLoadListener: function () {
    var self = this;
    $(".coverPageMedia img").bind("load", function(event) {
      self.updateImageSize(this);
    });
  },

  setup: function() {
    var self = this;
    this.registerImageLoadListener();
    this.registerResizeListener();
    this.scalePresentation();
    window.addEventListener('load', function () {
      self.presentationPageLoaded();
    }, false);
  },

  presentationPageLoaded: function () {
    this.disableAnchorTags();
    var self = this;
    $(".coverPageMedia").each(function () {
      var mediaElement = this;
      mediaElement.addEventListener('resize', function () {
        self.updateLayoutForMediaItem(mediaElement);
      }, false);
      mediaElement.addEventListener('load', function () {
        self.updateLayoutForMediaItem(mediaElement);
      }, false);

      self.updateLayoutForMediaItem(this);
    });
  },

  isElementOnScreen: function(element) {
    var docViewTop = $(window).scrollTop();
    var docViewBottom = docViewTop + $(window).height();
    var elemTop = $(element).offset().top;
    var elemBottom = elemTop + $(element).height();
    return ((elemBottom <= docViewBottom) && (elemTop >= docViewTop));
  },

  scrollIntoView: function(element) {
    $(window).scrollTop($(element).offset().top + ($(element).height() / 2) - ($(window).height() / 2))
  },

  getMediaPaths: function () {
    var paths = [];
    var self = this;
    $(".coverPageMedia [src]").each(function () {
      var path = this.getAttribute('src');
      if (path) {
        path = self.stripQuickLoopPathPrefix(path);
        paths.push(path);
      }
    });
    return paths;
  }

};

window.notePresenter = new ENCPPresenter();
$(document).on("ready", function() {
  window.notePresenter.setup();
});
